/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.clip;

import java.util.*;

import org.apache.log4j.Logger;

/**
 * An entity that monitors the system clipboard and notifies observers when
 * the contents have changed.
 *
 * @author  murali
 */
public class ClipboardCallback implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ClipboardCallback.class);

    public static final int SLEEP_TIME = 500;

    private static ClipboardCallback INSTANCE = new ClipboardCallback();



    //------------------------------------------------------------
    //- Class Functions

    public static void register(ClipboardListener pListener) {
        INSTANCE.addListener(pListener);
    }

    public static void deregister(ClipboardListener pListener) {
        INSTANCE.removeListener(pListener);
    }


    protected static void interrupt() {
        INSTANCE.getThread().interrupt();
    }


    //------------------------------------------------------------
    //- Instance Variables

    private String vLastContent = "";
    private final Collection<ClipboardListener> vListeners = new ArrayList<ClipboardListener>();
    private final Thread vThread;


    //------------------------------------------------------------
    //- Constructors

    private ClipboardCallback() {
        vThread = new Thread(this, "ClipboardCallback");
        vThread.setDaemon(true);
        vThread.start();
    }


    //------------------------------------------------------------
    //- Accessors

    private String getLastContent() {
        return vLastContent;
    }

    private Collection<ClipboardListener> getListeners() {
        return vListeners;
    }

    protected Thread getThread() {
        return vThread;
    }


    //------------------------------------------------------------
    //- Settors

    private void setLastContent(String pLastContent) {
        vLastContent = pLastContent;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected synchronized void addListener(ClipboardListener pListener) {
        getListeners().add(pListener);
    }

    protected synchronized void removeListener(ClipboardListener pListener) {
        getListeners().remove(pListener);
    }


    //------------------------------------------------------------
    //- Public Interface Functions



    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        //noinspection InfiniteLoopStatement
        while (true) {
            try {
                String current = ClipboardAccess.read();
                if (current.equals(getLastContent())) {
                    try {
                        Thread.sleep(SLEEP_TIME);
                    } catch (InterruptedException ie) {
                        // Nothing.
                    }
                } else {
                    synchronized (this) {
                        for (ClipboardListener clipboardListener : getListeners()) {
                            clipboardListener.newClipboardContents(current);
                        }
                    }

                    setLastContent(current);
                }
            } catch (Throwable thr) {
                LOG.error("Unhandled error in clipboard thread.", thr);
            }
        }
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
