/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.gui;

import org.apache.log4j.Logger;

import javax.swing.*;
import java.awt.*;

/**
 * Represent a single field (label + component) in a form.  It contains
 * the information on how to render the component in a GridBag.
 *
 * @author  murali
 */
public abstract class FormField {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(FormField.class);

    private static final Insets INSETS_MAIN = new Insets(2, 5, 2, 5);
    private static final Insets INSETS_INFO = new Insets(2, 0, 2, 0);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final JLabel vLabel;
    private final JComponent vComponent;
    private final double vVWeight;
    private final int vGridHeight;
    private final int vFill;
    private String vHelpText = "";


    //------------------------------------------------------------
    //- Constructors

    public FormField(String pString, JComponent pComponent, double pVWeight, int pGridHeight, int pFill) {
        vLabel = new JLabel(pString+":", SwingConstants.RIGHT);
        vComponent = pComponent;
        vVWeight = pVWeight;
        vGridHeight = pGridHeight;

        vLabel.setLabelFor(vComponent);
        vFill = pFill;
    }


    //------------------------------------------------------------
    //- Accessors

    public JLabel getLabel() {
        return vLabel;
    }

    public JComponent getComponent() {
        return vComponent;
    }

    public double getVWeight() {
        return vVWeight;
    }

    public int getGridHeight() {
        return vGridHeight;
    }

    public int getFill() {
        return vFill;
    }

    public String getHelpText() {
        return vHelpText;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setHelpText(String pHelpText) {
        vHelpText = pHelpText;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void addComponent(JPanel pPanel, GridBagConstraints gbc) {
        pPanel.add(getComponent(), gbc);
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void addTo(JPanel pPanel, GridBagConstraints gbc) {
        gbc.ipadx = 4;
        gbc.ipady = 4;
        gbc.insets = INSETS_MAIN;
        gbc.gridx = 0;
        gbc.weightx = 0;
        gbc.weighty = 0;
        gbc.gridheight = 1;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        pPanel.add(getLabel(), gbc);

        gbc.gridx = 1;
        gbc.weightx = 1;
        gbc.weighty = getVWeight();
        gbc.gridheight = getGridHeight();
        gbc.fill = getFill();
        addComponent(pPanel, gbc);

        if (!"".equals(getHelpText())) {
            gbc.ipadx = 0;
            gbc.ipady = 0;
            gbc.insets = INSETS_INFO;
            gbc.gridx = 2;
            gbc.weightx = 0;
            gbc.weighty = 0;
            gbc.gridheight = 1;
            gbc.fill = GridBagConstraints.NONE;

            InfoButton info = new InfoButton(null, getLabel().getText(), getHelpText());
            pPanel.add(info, gbc);
        }
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    protected abstract void setValue(Object pObject);
    protected abstract Object getValue();


    //------------------------------------------------------------
    //- Inner Classes

    public static class Generic extends FormField {

        public Generic(String pString, FormComponent pComponent) {
            super (pString, pComponent.toComponent(), 0, 1, GridBagConstraints.HORIZONTAL);
        }

        protected Object getValue() {
            FormComponent component = (FormComponent) getComponent();
            Object result = component.getValue();
            return result;
        }

        protected void setValue(Object pObject) {
            FormComponent component = (FormComponent) getComponent();
            component.setValue(pObject);
        }
    }


    public static class Field extends FormField {

        public Field(String pString, JTextField pTextField) {
            super (pString, pTextField, 0, 1, GridBagConstraints.HORIZONTAL);
        }

        protected Object getValue() {
            JTextField field = (JTextField) getComponent();
            Object result = field.getText();
            return result;
        }

        protected void setValue(Object pObject) {
            JTextField field = (JTextField) getComponent();
            field.setText(String.valueOf(pObject));
        }
    }


    public static class Area extends FormField {

        public Area(String pString, JTextArea pTextarea) {
            super (pString, pTextarea, 1, pTextarea.getRows(), GridBagConstraints.BOTH);
        }

        protected Object getValue() {
            JTextArea area = (JTextArea) getComponent();
            Object result = area.getText();
            return result;
        }

        protected void setValue(Object pObject) {
            JTextArea area = (JTextArea)getComponent();
            area.setText(String.valueOf(pObject));
            area.setCaretPosition(0);
        }

        protected void addComponent(JPanel pPanel, GridBagConstraints gbc) {
            pPanel.add(new JScrollPane(getComponent()), gbc);
        }

    }


    public static class Html extends FormField {

        public Html(String pString, JEditorPane pEditorPane) {
            super (pString, pEditorPane, 1, 4, GridBagConstraints.BOTH);
        }

        protected Object getValue() {
            JEditorPane html = (JEditorPane) getComponent();
            Object result = html.getText();
            return result;
        }

        protected void setValue(Object pObject) {
            JEditorPane html = (JEditorPane)getComponent();
            html.setText(String.valueOf(pObject));
            html.setCaretPosition(0);
        }

        protected void addComponent(JPanel pPanel, GridBagConstraints gbc) {
            pPanel.add(new JScrollPane(getComponent()), gbc);
        }

    }


    public static class Choice extends FormField {

        public Choice(String pString, JComboBox pComboBox) {
            super (pString, pComboBox, 0, 1, GridBagConstraints.NONE);
        }

        protected Object getValue() {
            JComboBox choice = (JComboBox) getComponent();
            Object result = choice.getSelectedItem();
            return result;
        }

        protected void setValue(Object pObject) {
            JComboBox choice = (JComboBox) getComponent();
            choice.setSelectedItem(pObject);
        }

    }


    public static class List extends FormField {

        public List(String pString, JList pList) {
            super (pString, pList, 1, 4, GridBagConstraints.BOTH);
        }

        protected Object getValue() {
            JList list = (JList) getComponent();
            Object result = list.getSelectedValue();
            return result;
        }

        protected void setValue(Object pObject) {
            JList list = (JList) getComponent();
            list.setSelectedValue(pObject, true);
        }

        protected void addComponent(JPanel pPanel, GridBagConstraints gbc) {
            pPanel.add(new JScrollPane(getComponent()), gbc);
        }

    }


    //------------------------------------------------------------
    //- Main



}
