/*
   Copyright 2006-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.util.*;

import org.apache.log4j.Logger;

/**
 * Basically an infinite loop with a sleep period (to avoid CPU
 * saturation).  It is used to continuously execute one or more
 * given Runnables.  An instance can either be passed to a Thread
 * (it is a Runnable) or it can spawn its own Thread.  Once started,
 * it can be suspended (disabled), resumed (enabled), and stopped.
 *
 * @author  murali
 */
public class ContinuousExecution implements Runnable {

    //------------------------------------------------------------
    //- Class Variables

    protected static final Logger LOG = Logger.getLogger(ContinuousExecution.class);


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private boolean vIsAlive = true;
    private boolean vIsEnabled = true;
    private int vPeriod = 1000;
    private final Collection<Runnable> fChildren = new ArrayList<Runnable>();


    //------------------------------------------------------------
    //- Constructors

    public ContinuousExecution(int pPeriod) {
        setPeriod(pPeriod);
    }


    //------------------------------------------------------------
    //- Accessors

    public boolean getIsAlive() {
        return vIsAlive;
    }

    public boolean getIsEnabled() {
        return vIsEnabled;
    }

    public int getPeriod() {
        return vPeriod;
    }

    protected Collection<Runnable> getChildren() {
        return fChildren;
    }


    //------------------------------------------------------------
    //- Settors

    protected void setIsAlive(boolean pIsAlive) {
        vIsAlive = pIsAlive;
    }

    protected void setIsEnabled(boolean pIsEnabled) {
        vIsEnabled = pIsEnabled;
    }

    public void setPeriod(int pPeriod) {
        vPeriod = pPeriod;
    }


    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    protected void runChildren() {
        for (Runnable runnable : getChildren()) {
            ThreadMonitor.update("Fired.");
            runnable.run();
            ThreadMonitor.update("Finished.");
        }
    }


    //------------------------------------------------------------
    //- Public Interface Functions

    public void add(Runnable pChild) {
        getChildren().add(pChild);
    }


    public void startThread(String pName) {
        Thread thread = new Thread(this, pName);
        thread.setDaemon(true);
        thread.start();
    }


    public void suspend() {
        setIsEnabled(false);
    }

    public void resume() {
        setIsEnabled(true);
    }

    public void stop() {
        setIsEnabled(false);
        setIsAlive(false);
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public void run() {
        do {
            if (getIsEnabled()) {
                try {
                    runChildren();
                } catch (Throwable thr) {
                    LOG.error("Fatal error.", thr);
                }
            }

            try {
                Thread.sleep(getPeriod());
            } catch (InterruptedException ie) {
                // Do nothing.
            }
        } while (getIsAlive());
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main



}
