/*
   Copyright 2008-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import org.apache.log4j.Logger;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.KeyEvent;

/**
 * An EventQueue that keeps track of last activity, so it can be determined
 * how long the user is idle.
 * 
 * http://www.javakb.com/Uwe/Forum.aspx/java-gui/2628/Inactivity-detect
 *
 * User: murali
 * Date: Aug 29, 2008
 * Time: 1:02:12 PM
 */
public class IdleMonitorEventQueue extends EventQueue {

    //------------------------------------------------------------
    //- Class Variables

    @SuppressWarnings({"UnusedDeclaration"})
    private static final Logger LOG = Logger.getLogger("com.notuvy.util");

    private static final IdleMonitorEventQueue INSTANCE = new IdleMonitorEventQueue();

    //------------------------------------------------------------
    //- Class Functions

    public static IdleMonitorEventQueue getInstance() {
        return INSTANCE;
    }

    /**
     * Use this singleton as the current system EventQueue.
     */
    public static synchronized void register() {
        EventQueue current = Toolkit.getDefaultToolkit().getSystemEventQueue();
        if (current != INSTANCE) {
            current.push(INSTANCE);
        }
    }

    /**
     * The time of the last UI activity.
     * 
     * @return  time as milliseconds.
     */
    public static long getLastActivity() {
        return INSTANCE.fLastActivity;
    }

    public static void touch() {
        INSTANCE.markActive();
    }

    //------------------------------------------------------------
    //- Instance Variables

    private long fLastActivity;

    //------------------------------------------------------------
    //- Constructors

    private IdleMonitorEventQueue() {
    }

    //------------------------------------------------------------
    //- Getters

    //------------------------------------------------------------
    //- Settors

    //------------------------------------------------------------
    //- Private/Protected Utility Functions

    private void markActive() {
        fLastActivity = WallClock.currentTimeMillis();
    }

    //------------------------------------------------------------
    //- Public Interface Functions

    //------------------------------------------------------------
    //- Class Interface Functions

    protected void dispatchEvent(AWTEvent event) {
        try {
            super.dispatchEvent(event);
        } catch (RuntimeException re) {
            LOG.error(String.format("UI error dispatching [%s].", event.toString()), re);
            throw re;
        }

        // Note: maybe we want KEY_PRESSED, because when a button launches an
        // action that consumes thread, the release does not register until after
        // the action.
        switch (event.getID()) {
            case MouseEvent.MOUSE_RELEASED:
            case KeyEvent.KEY_RELEASED:
                markActive();
                break;
        }
    }

    //------------------------------------------------------------
    //- Inner Classes

//    static class Tester extends MainFrame {
//
//        Tester() {
//            super (null);
//        }
//        protected void initializeUIComponents() {
//        }
//
//        protected void buildUI(Container pContainer) {
//            pContainer.add(new JButton("button"));
//        }
//
//        public static void main(String[] args) {
//            enact();
//            new Tester().launch();
//        }
//    }

    //------------------------------------------------------------
    //- Main

}
