/*
   Copyright 2007-2009 Murali Krishnan

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
 */

package com.notuvy.util;

import java.io.*;
import java.util.jar.Manifest;
import java.util.jar.Attributes;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.net.URL;

import org.apache.log4j.Logger;
import org.apache.commons.lang.StringUtils;

/**
 * A utilty that extracts metadata from the JAR manifest.  It is created by passing
 * in a class from the desired jar (since there could be multiple jars on the classpath,
 * and thus multiple manifests).
 *
 * @author  murali
 */
public class ManifestProperties {

    //------------------------------------------------------------
    //- Class Variables

    private static final String RESOURCE_URI = "/META-INF/MANIFEST.MF";

    public static final String UNDEFINED = "undefined";

    private static final Pattern P_FORMAT = Pattern.compile("^(\\d{2})");

    private static final FixedWidthText LINE_BREAKER = new FixedWidthText(80);

    protected static final Logger LOG = Logger.getLogger("com.notuvy.util");


    //------------------------------------------------------------
    //- Class Functions


    //------------------------------------------------------------
    //- Instance Variables

    private final Translation<String> fMap = new Translation<String>()
            .addTransform(Translation.CASE_INSENSITIVE)
            .setFallback(UNDEFINED);

    private String fLicense = "";

    //------------------------------------------------------------
    //- Constructors

    public ManifestProperties(Class pReferenceClass) {
        String classContainer = pReferenceClass.getProtectionDomain().getCodeSource().getLocation().toString();
        try {
            URL manifestUrl = new URL("jar:" + classContainer + "!" + RESOURCE_URI);
            Manifest manifest = new Manifest(manifestUrl.openStream());
            Attributes mainAttributes = manifest.getMainAttributes();
            for (Object o : mainAttributes.keySet()) {
                // LOG.debug("[" + o + "]: [" + mainAttributes.get(o) + "]");
                String value = (String) mainAttributes.get(o);
                if (StringUtils.isNotBlank(value)) {
                    String key = String.valueOf(o);
                    String val = "ProjectDescription".equals(key) ? LINE_BREAKER.process(value) : value;
                    fMap.define(key, val);
                }
            }
        } catch (IOException ioe) {
            LOG.error("Problem reading properties.", ioe);
        }

        try {
            StringBuilder builder = new StringBuilder();
            URL licenseUrl = new URL("jar:" + classContainer + "!/LICENSE.txt");
            InputStream is = licenseUrl.openStream();
            InputStreamReader isr = new InputStreamReader(is);
            BufferedReader br = new BufferedReader(isr);

            for (String line=br.readLine(); line!=null; line=br.readLine()) {
                builder.append(line).append("\n");
            }

            br.close();
            isr.close();
            is.close();

            fLicense = builder.toString();
        } catch (FileNotFoundException fnfe) {
            LOG.error(fnfe);
        } catch (IOException ioe) {
            LOG.error("Problem reading LICENSE.txt", ioe);
        }
    }


    //------------------------------------------------------------
    //- Accessors

    public String getProjectName() {
        return fMap.lookup("ProjectName");
    }

    public String getProjectVersion() {
        return fMap.lookup("ProjectVersion");
    }

    public String getBuildTime() {
        return fMap.lookup("BuildTime");
    }

    public String getProjectDescription() {
        return fMap.lookup("ProjectDescription");
    }

    public String getLicense() {
        return fLicense;
    }

    //------------------------------------------------------------
    //- Settors



    //------------------------------------------------------------
    //- Private/Protected Utility Functions


    //------------------------------------------------------------
    //- Public Interface Functions

    public String toCopyrightString() {
        StringBuilder builder = new StringBuilder("Copyright (c) ");
        String copyrightStart = fMap.lookup("CopyrightStart");
        builder.append(copyrightStart);
        String buildTime = getBuildTime();
        if (StringUtils.isNotBlank(buildTime)) {
            Matcher matcher = P_FORMAT.matcher(buildTime);
            if (matcher.find()) {
                String copyrightEnd = "20" + matcher.group(1);
                if (!copyrightEnd.equals(copyrightStart)) {
                    builder.append("-").append(copyrightEnd);
                }
            }
        }
        builder.append(" ").append(fMap.lookup("CopyrightHolder")).append(". All rights reserved.");
        return builder.toString();
    }

    public String toProjectSummaryString() {
        StringBuilder builder = new StringBuilder();
        builder.append(getProjectName())
                .append(", version ").append(getProjectVersion())
                .append(" [").append(getBuildTime()).append("]");
        return builder.toString();
    }

    public String toAboutString() {
        StringBuilder builder = new StringBuilder();

        builder.append(toProjectSummaryString()).append("\n\n");
        builder.append(getProjectDescription()).append("\n\n");
        builder.append(toCopyrightString()).append("\n\n");
        builder.append(getLicense());

        return builder.toString();
    }


    //------------------------------------------------------------
    //- Class Interface Functions

    public String toString() {
        return toProjectSummaryString();
    }


    //------------------------------------------------------------
    //- Inner Classes



    //------------------------------------------------------------
    //- Main

    public static void main(String[] args) {
        try {
            ManifestProperties mp = new ManifestProperties(ManifestProperties.class);
            LOG.info(mp);
            LOG.info(mp.toCopyrightString());
            LOG.info(mp.getProjectDescription());
        } catch (Throwable thr) {
            LOG.error("Fatal error.", thr);
        }
    }


}